@@ -5,7 +5,7 @@ use arrayvec::ArrayVec;
 
 use crate::run_block_av_flat::{VC_LEN, MAX_BLOCK_SIZE, RLEBlock}; // current best
 
-const MAX_NODE_SIZE: usize = 64; // must be even
+const MAX_NODE_SIZE: usize = 64; // must be a power of 2
 const NODE_MIDPOINT: usize = MAX_NODE_SIZE / 2;
 
 /// A B+ tree structure that has special functionality to encode runs of the same symbol in a defined alphabet
@@ -33,13 +33,54 @@ struct RLEBPlusNode {
     /// stores the individual symbol counts for that child node and all its children
     total_symbols: ArrayVec<[u64; VC_LEN], MAX_NODE_SIZE>,
     /// the indices of any child nodes; if it's a leaf this refers to RLEBlocks, otherwise other RLEBPlusNodes
-    children: Vec<usize>
+    children: ArrayVec<usize, MAX_NODE_SIZE>
+}
+
+impl RLEBPlusNode {
+    #[inline]
+    fn count(&self, relative_index: u64, symbol: u8) -> (usize, u64, u64) {
+        let mut total_count: u64 = 0;
+        let bs_index: usize = self.total_counts.iter().position(|&v| {
+            total_count += v;
+            total_count >= relative_index
+        }).unwrap();
+        total_count -= self.total_counts[bs_index];
+
+        let symbol_count: u64 = self.total_symbols[..bs_index].iter().fold(0, |acc, v| acc + v[symbol as usize]);
+        (bs_index, symbol_count, total_count)
+    }
+
+    //for now, this is a clone except for two minor increments on the found node
+    #[inline]
+    fn insert_and_count(&mut self, relative_index: u64, symbol: u8) -> (usize, u64, u64) {
+        let mut total_count: u64 = 0;
+        let bs_index: usize = self.total_counts.iter().position(|&v| {
+            total_count += v;
+            total_count >= relative_index
+        }).unwrap();
+        total_count -= self.total_counts[bs_index];
+
+        let symbol_count: u64 = self.total_symbols[..bs_index].iter().fold(0, |acc, v| acc + v[symbol as usize]);
+
+        //increment total counts and symbols for this node
+        self.total_counts[bs_index] += 1;
+        self.total_symbols[bs_index][symbol as usize] += 1;
+
+        (bs_index, symbol_count, total_count)
+    }
+
+    #[inline]
+    fn build_indices(&mut self) {
+        //place-holder
+    }
 }
 
 impl Default for RLEBPlusTree {
     #[inline]
     fn default() -> Self {
         let children: Vec<RLEBlock> = vec![Default::default()];
+        let mut child_indices: ArrayVec<usize, MAX_NODE_SIZE> = Default::default();
+        child_indices.push(0);
         let mut total_counts = ArrayVec::<u64, MAX_NODE_SIZE>::new();
         total_counts.push(0);
         let mut total_symbols = ArrayVec::<[u64; VC_LEN], MAX_NODE_SIZE>::new();
@@ -49,7 +90,7 @@ impl Default for RLEBPlusTree {
             parent: 0,
             total_counts,
             total_symbols,
-            children: vec![0]
+            children: child_indices
         };
 
         RLEBPlusTree {
@@ -114,36 +155,34 @@ impl RLEBPlusTree {
         let mut current_node: &RLEBPlusNode = &self.nodes[current_node_index];
         let mut relative_index: u64 = index;
         let mut total_count: u64 = 0;
-        let mut tc: u64;
             
         //iterate downwards until we find a leaf node point to run blocks
         while !current_node.is_leaf {
             //linear search tended to be faster in practice
-            tc = 0;
-            let bs_index: usize = current_node.total_counts.iter().position(|v| {
-                tc += *v;
-                tc >= relative_index
-            }).unwrap();
-            tc -= current_node.total_counts[bs_index];
-            relative_index -= tc;
-            total_count += current_node.total_symbols[..bs_index].iter().fold(0, |acc, v| acc + v[value as usize]);
+            let (bs_index, sc, tc) = current_node.count(relative_index, value);
 
+            //add symbol count to the running total
+            total_count += sc;
+            //adjust our relative index by how much we went through
+            relative_index -= tc;
+
+            //go down to the indexing node
             current_node_index = current_node.children[bs_index];
             current_node = &self.nodes[current_node_index];
         }
 
-        //we're in a leaf
-        tc = 0;
-        let bs_index: usize = current_node.total_counts.iter().position(|v| {
-            tc += *v;
-            tc >= relative_index
-        }).unwrap();
-        tc -= current_node.total_counts[bs_index];
+        //we're in a leaf, still need to search for the data node
+        let (bs_index, sc, tc) = current_node.count(relative_index, value);
+        
+        //add symbol count to the running total
+        total_count += sc;
+        //adjust our relative index by how much we went through
         relative_index -= tc;
+
+        //get the data node from the search index
         let arena_index: usize = current_node.children[bs_index];
         
         //add in the final counts and return
-        total_count += current_node.total_symbols[..bs_index].iter().fold(0, |acc, v| acc + v[value as usize]);
         total_count += self.data_arena[arena_index].count(relative_index, value);
         total_count
     }
@@ -174,25 +213,16 @@ impl RLEBPlusTree {
         let mut current_node = &mut self.nodes[current_node_index];
         let mut relative_index: u64 = index;
         let mut total_count: u64 = 0;
-        let mut tc: u64;
 
         //iterate downwards until we find a leaf node point to run blocks
         while !current_node.is_leaf {
-            //get the insertion index
-            tc = 0;
-            let bs_index: usize = current_node.total_counts.iter().position(|v| {
-                tc += *v;
-                tc >= relative_index
-            }).unwrap();
-            tc -= current_node.total_counts[bs_index];
-            
-            //subtract out the before counts for the relative index, and also add them to our total for return
-            relative_index -= tc;
-            total_count += current_node.total_symbols[..bs_index].iter().fold(0, |acc, v| acc + v[value as usize]);
+            //linear search tended to be faster in practice
+            let (bs_index, sc, tc) = current_node.insert_and_count(relative_index, value);
 
-            //increment total counts and symbols for this node
-            current_node.total_counts[bs_index] += 1;
-            current_node.total_symbols[bs_index][value as usize] += 1;
+            //add symbol count to the running total
+            total_count += sc;
+            //adjust our relative index by how much we went through
+            relative_index -= tc;
 
             //update current node
             current_node_index = current_node.children[bs_index];
@@ -200,24 +230,15 @@ impl RLEBPlusTree {
         }
 
         //we're in a leaf, get the leaf index and then the arena index for the data node
-        tc = 0;
-        let bs_index: usize = current_node.total_counts.iter().position(|v| {
-            tc += *v;
-            tc >= relative_index
-        }).unwrap();
-        tc -= current_node.total_counts[bs_index];
+        let (bs_index, sc, tc) = current_node.insert_and_count(relative_index, value);
         
-        let arena_index: usize = current_node.children[bs_index];
-
-        //subtract out the before counts for the relative index, and also add them to our total for return
+        //add symbol count to the running total
+        total_count += sc;
+        //adjust our relative index by how much we went through
         relative_index -= tc;
-        total_count += current_node.total_symbols[..bs_index].iter().fold(0, |acc, v| acc + v[value as usize]);
-
-        //increment total counts and symbols for this node
-        current_node.total_counts[bs_index] += 1;
-        current_node.total_symbols[bs_index][value as usize] += 1;
-
+        
         //increment the total count by what comes back from the block insertion
+        let arena_index: usize = current_node.children[bs_index];
         total_count += self.data_arena[arena_index].insert_and_count(relative_index, value);
         
         //handle any internal splitting that needs to occur due to nodes filling up
@@ -254,6 +275,8 @@ impl RLEBPlusTree {
             //insert the child into the list
             current_node.children.insert(bs_index+1, new_arena_index);
 
+            current_node.build_indices();
+
             //recursively push as needed into internal nodes
             while self.nodes[current_node_index].children.len() == MAX_NODE_SIZE {
                 assert!(self.nodes[current_node_index].total_counts.len() == MAX_NODE_SIZE);
@@ -275,7 +298,7 @@ impl RLEBPlusTree {
                 }
 
                 //right child is create from the removed data
-                let right_child = RLEBPlusNode {
+                let mut right_child = RLEBPlusNode {
                     is_leaf: current_node.is_leaf,
                     parent: current_node.parent,
                     total_counts: rtc,
@@ -283,6 +306,10 @@ impl RLEBPlusTree {
                     children: rtch
                 };
 
+                //all node changes have been made to current & right at this point
+                current_node.build_indices();
+                right_child.build_indices();
+
                 //get the total counts for the new right node
                 let right_total_count: u64 = right_child.total_counts.iter().sum();
                 let mut right_total_symbols: [u64; VC_LEN] = [0; VC_LEN];
@@ -328,6 +355,10 @@ impl RLEBPlusTree {
                     self.nodes[0].children[0] = left_child_index;
                     self.nodes[0].children[1] = right_child_index;
                     self.nodes[0].children.truncate(2);
+                    
+                    //root node changes are complete
+                    self.nodes[0].build_indices();
+
                 } else {
                     //this is not root, so left child is completely done at this point; we need to insert right child though
                     //push the midpoint before counts into the parent and add the new (right) child
@@ -351,6 +382,9 @@ impl RLEBPlusTree {
                     parent_node.total_symbols[child_index] = left_total_symbols;
                     parent_node.total_symbols.insert(child_index+1, right_total_symbols);
 
+                    //parent node changes are complete
+                    parent_node.build_indices();
+
                     //finally, push the right node as a new node
                     self.nodes.push(right_child);
 
@@ -418,7 +452,7 @@ impl RLEBPlusTree {
     /// ```
     pub fn run_iter(&self) -> RLEBPlusTreeRunIterator<'_> {
         let next_child_index = self.next_child[0];
-        let current_block_iter = self.data_arena[0].raw_iter();
+        let current_block_iter = Box::new(self.data_arena[0].run_iter());
         RLEBPlusTreeRunIterator {
             tree: self,
             next_child_index,
@@ -468,7 +502,7 @@ impl<'a> Iterator for RLEBPlusTreeIterator<'a> {
 pub struct RLEBPlusTreeRunIterator<'a> {
     tree: &'a RLEBPlusTree,
     next_child_index: usize,
-    current_block_iter: std::slice::Iter<'a, (u8, u8)>,
+    current_block_iter: Box<dyn Iterator<Item=(u8, u64)> + 'a>, //std::slice::Iter<'a, (u8, u64)>,
     next_sym: u8,
     next_count: u64
 }
@@ -477,6 +511,7 @@ impl<'a> Iterator for RLEBPlusTreeRunIterator<'a> {
     type Item = (u8, u64);
     /// Will return the next run contained by the compressed B+ tree data
     fn next(&mut self) -> Option<(u8, u64)> {
+        //None
         loop {
             let next_pair = match self.current_block_iter.next() {
                 //current block has data, so return it
@@ -488,7 +523,7 @@ impl<'a> Iterator for RLEBPlusTreeRunIterator<'a> {
                         None
                     } else {
                         //get the next block iterator and update the next child index before returning a value
-                        self.current_block_iter = self.tree.data_arena[self.next_child_index].raw_iter();
+                        self.current_block_iter = Box::new(self.tree.data_arena[self.next_child_index].run_iter());
                         self.next_child_index = self.tree.next_child[self.next_child_index];
 
                         //this *should* always work given our current setup, there shouldn't be any empty
@@ -499,7 +534,7 @@ impl<'a> Iterator for RLEBPlusTreeRunIterator<'a> {
             };
 
             match next_pair {
-                Some(&pair_values) => {
+                Some(pair_values) => {
                     //check if this run matches the ongoing run
                     if pair_values.0 == self.next_sym {
                         //part of the same run, increment and loop back
